package com.googlecode.functionalcollections;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.NoSuchElementException;
import java.util.concurrent.atomic.AtomicInteger;

import org.junit.Test;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;

public class FunctionalIterablesTest {
	
	@Test
	public void each() {
		final Collection<Object> source = new ArrayList<Object>();
		source.add(new Object());
		source.add(new Object());
		
		final AtomicInteger counter = new AtomicInteger();
		
		FunctionalIterables.make(source).each(new Block<Object>() {
			public void apply(Object input) {
				assertTrue(source.contains(input));
				counter.incrementAndGet();
			}
		});
		
		assertTrue(source.size() == counter.intValue());
	}
	
	@Test
	public void any() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		
		boolean any = FunctionalIterables.make(source).any(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj1;
			}
		});
		
		assertTrue(any);
	}
	
	@Test
	public void all() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		
		boolean all = FunctionalIterables.make(source).any(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj1 || input == obj2;
			}
		});
		
		assertTrue(all);
	}
	
	@Test
	public void findSuccess() {
		final Object obj = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj);
		
		
		Object found = FunctionalIterables.make(source).find(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj;
			}
		});
		
		assertEquals(obj, found);
	}
	
	@Test(expected = NoSuchElementException.class)
	public void findFailed() {
		Collection<Object> source = new ArrayList<Object>();

		FunctionalIterables.make(source).find(new Predicate<Object>() {
			public boolean apply(Object input) {
				return false;
			}
		});
	}
	
	@Test
	public void detectSuccess() {
		final Object obj = new Object();
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj);
		
		
		Object found = FunctionalIterables.make(source).detect(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj;
			}
		});
		
		assertEquals(obj, found);
	}
	
	@Test
	public void detectFailed() {
		Collection<Object> source = new ArrayList<Object>();

		Object detected = FunctionalIterables.make(source).detect(new Predicate<Object>() {
			public boolean apply(Object input) {
				return false;
			}
		});
		
		assertNull(detected);
	}
	
	@Test
	public void filter() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		final Object obj3 = new Object();
		final Object obj4 = new Object();
		
		Collection<Object> expected = new ArrayList<Object>();
		expected.add(obj1);
		expected.add(obj3);
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		source.add(obj4);
		
		FunctionalIterable<Object> filtered = FunctionalIterables.make(source).filter(new Predicate<Object>() {
			public boolean apply(Object input) {
				return input == obj1 || input == obj3;
			}
		});
		
		assertTrue(Iterables.elementsEqual(expected, filtered));
	}
	
	@Test
	public void transform() {
		final Object obj1 = new Object();
		final Object obj2 = new Object();
		final Object obj3 = new Object();
		final Object obj4 = new Object();
		
		Collection<String> expected = new ArrayList<String>();
		expected.add(obj1.toString());
		expected.add(obj2.toString());
		expected.add(obj3.toString());
		expected.add(obj4.toString());
		
		Collection<Object> source = new ArrayList<Object>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		source.add(obj4);
		
		FunctionalIterable<String> transformed = FunctionalIterables.make(source).transform(new Function<Object, String>() {
			public String apply(Object from) {
				return from.toString();
			}
		});
		
		assertTrue(Iterables.elementsEqual(expected, transformed));
	}
	
	@Test
	public void inject() {
		final String obj1 = "obj1";
		final String obj2 = "obj2";
		final String obj3 = "obj3";
		
		Collection<String> source = new ArrayList<String>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		
		Integer initial = 0;
		
		Integer totalLength = FunctionalIterables.make(source).inject(initial, new Injector<Integer, String>() {
			public Integer apply(Integer memo, String input) {
				return memo + input.length();
			}
		});
	
		assertTrue(totalLength == (obj1.length() + obj2.length() + obj3.length()));
	}
	
	@Test
	public void toCollection() {
		final String obj1 = "obj1";
		final String obj2 = "obj2";
		final String obj3 = "obj3";
		
		Collection<String> source = new ArrayList<String>();
		source.add(obj1);
		source.add(obj2);
		source.add(obj3);
		
		Collection<String> dest = FunctionalIterables.make(source).toCollection();
	
		assertTrue(Iterables.elementsEqual(source, dest));
	}
	
}
